#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "my_matrix.h"

void dtrsv_(char *UPLO, char *TRANS, char *DIAG, INT * N, double *A, INT *LDA, double *RHS, INT *INCX); 
void dtrsm_(char *SIDE, char *UPLO, char *TRANS, char *DIAG, INT *M, INT *N, double *alpha, double *A, INT *LDA, double *RHS, INT *LDB); 


void l_solvev(struct my_matrix_st *L, double *rhs) {
	INT i,k; 

	for ( i = 0; i<L->rows; i++ ) {
		for ( k =0; k< i-1; k++ ) {
			rhs[i] -= L->values[L->LD*k+i] * rhs[k]; 
		}
		rhs[i] /= L->values[L->LD*i+i]; 
	}

}

void l_solvem(struct my_matrix_st *L,struct my_matrix_st *rhs){
	INT i; 
	
	for ( i =0 ; i < rhs->cols; i++ ) {
		l_solvev(L, &rhs->values[i*rhs->LD]); 
	}
}

void l_solvem_trsv(struct my_matrix_st *L,struct my_matrix_st *rhs){
	INT i; 
	INT one=1; 
	
	for ( i =0 ; i < rhs->cols; i++ ) {
		dtrsv_("L","N","N",&L->rows, L->values, &L->LD, rhs->values+rhs->LD*i, &one); 
	}
}

void l_solvem_trsm(struct my_matrix_st *L,struct my_matrix_st *rhs){
	INT i;
	double alpha=1; 

	dtrsm_("L","L","N","N",&L->rows, &rhs->cols,&alpha,L->values, &L->LD, rhs->values, &rhs->LD); 
}


int main (int argc , char *argv ){
	struct my_matrix_st A; 
	struct my_matrix_st RHS; 
	double tic,toc; 

	// read a dense matrix to A and measure the time
	tic = wtime(); 
	if ( my_matrix_read ("demo5625.mtx", &A)){
		fprintf(stderr,"Can not read matrix.\n"); 
		return 1; 
	}
	toc = wtime(); 
	printf("Reading the matrix took %lg seconds\n", toc - tic);
	// my_matrix_print(&A);

	// Your code comes here.
	my_matrix_rand(&RHS,A.rows, 100); 	

	tic = wtime(); 
	l_solvem(&A, &RHS); 
	toc = wtime(); 

	printf("Naive solve took %lg seconds \n", toc - tic);

	tic = wtime(); 
	l_solvem_trsv(&A, &RHS); 
	toc = wtime(); 

	printf("DTRSV solve took %lg seconds \n", toc - tic);

	tic = wtime(); 
	l_solvem_trsm(&A, &RHS); 
	toc = wtime(); 

	printf("DTRSM solve took %lg seconds \n", toc - tic);




	// Cleanup
	my_matrix_clear(&A); 
	my_matrix_clear(&RHS); 
	return 0; 

}
